import multiprocessing as mp
from multiprocessing import Pool
import mandelbrot
from time import time
from PIL import Image
import os
from collections import defaultdict

# Set the size of the image
width, height = (280 * 2, 300 * 2)

# Get the count of processors
cpu_count = mp.cpu_count()

# Set the size of the blocks
block_width = 70
block_height = 60
# How many blocks in a row?
blocks_per_row = width / block_width

def compute_block(block_index):
    mandelbrot.set_size((width, height))

    start = time()
    result = []

    # Compute the upper-left corner of the current block
    start_y = int(block_index / blocks_per_row) * block_height
    start_x = int(block_index % blocks_per_row) * block_width

    # Only compute my block area
    for y in range(start_y, start_y + block_height):
        for x in range(start_x, start_x + block_width):
            pixel = mandelbrot.compute_pixel((x, y))
            result.append(pixel)
    end = time()

    # Return the ID of my process, timing info and the actual
    # computed pixels for the current block
    return (os.getpid(), end - start, result)

def create_image():
    # Diagnostic info
    timing = defaultdict(lambda: 0)
    blocks = defaultdict(lambda: 0)

    # Create an image
    output = Image.new("RGB", (width, height))

    # Create a pool with a process per processor
    pool = Pool(processes=cpu_count)

    # How many blocks in total?
    total_blocks = int((width / block_width) * (height / block_height))

    start = time()
    # Call the map function on the pool, iterating on the block indeces
    pixel_blocks = pool.map(compute_block, range(total_blocks))
    end = time()
    print("It took {:.4} seconds".format(end - start))

    # Merge all the results together
    for block_index, (pid, proc_time, pixel_block) in enumerate(pixel_blocks):
        timing[pid] += proc_time
        blocks[pid] += 1

        # The results are received as a plain array, compute the correct
        # index where they should be placed
        x = int(block_index % blocks_per_row) * block_width
        y = int(block_index / blocks_per_row) * block_height
        for pixel in pixel_block:
            output.putpixel((x, y), pixel)

            x += 1
            if x % block_width == 0:
                x -= block_width
                y += 1

    # Save the image
    output.save("/tmp/out.png")

    print("\n".join(map(lambda p: "Process {} took {:.4} seconds ({} blocks)".format(p[0], timing[p[1]], blocks[p[1]]), enumerate(timing))))

def main():
    create_image()

if __name__ == "__main__":
    main()
