import multiprocessing as mp
from multiprocessing import Process, Pool, Array
import mandelbrot
from time import time
from PIL import Image
import os
from collections import defaultdict

# Set the size of the image
width, height = (280 * 2, 300 * 2)

# Get the count of processors
cpu_count = mp.cpu_count()

# Set the size of the blocks
block_width = 70
block_height = 60
# How many blocks in a row?
blocks_per_row = width / block_width

image_data = None

# This function is called for every process in the pool once
# It simply saves in a global variable a reference to the
# shared memory area
def init_process(shared_array):
    global image_data
    image_data = shared_array

def compute_block(block_index):
    mandelbrot.set_size((width, height))

    start = time()

    # Compute the upper-left corner of the current block
    start_y = int(block_index / blocks_per_row) * block_height
    start_x = int(block_index % blocks_per_row) * block_width

    # Only compute my block area
    for y in range(start_y, start_y + block_height):
        for x in range(start_x, start_x + block_width):
            pixel = mandelbrot.compute_pixel((x, y))
            linear_index = 3 * (x + y * width)

            # Directly write the result in the shared memory,
            # one entry per color channel (RGB)5B
            image_data[linear_index + 0] = pixel[0]
            image_data[linear_index + 1] = pixel[1]
            image_data[linear_index + 2] = pixel[2]
    end = time()

    return (os.getpid(), end - start)

def create_image():
    # Diagnostic info
    timing = defaultdict(lambda: 0)
    blocks = defaultdict(lambda: 0)

    # Create image
    output = Image.new("RGB", (width, height))

    # Create the shared memory array, with an element for each
    # pixel of the image and for each color channel (RGB)
    # Try it without lock=False
    shared_array = Array("i", [0] * width * height * 3, lock=False)

    # Create the process Pool, pass the shared memory to its
    # initializer
    pool = Pool(processes=cpu_count,initializer=init_process,initargs=(shared_array,))

    # How many blocks?
    total_blocks = int((width / block_width) * (height / block_height))

    start = time()
    # Use the map function on the blocks
    results = pool.map(compute_block, range(total_blocks))
    end = time()
    print("It took {:.4} seconds".format(end - start))

    # Diagnostic info
    for pid, proc_time in results:
        timing[pid] += proc_time
        blocks[pid] += 1

    # Rebuild the image accessing the shared memory array
    for i in range(0, width * height * 3, 3):
        r, g, b = shared_array[i:i + 3]
        i = int(i / 3)
        x = int(i % width)
        y = int(i / width)
        output.putpixel((x, y), (r, g, b))

    # Save the image
    output.save("/tmp/out.png")

    print("\n".join(map(lambda p: "Process {} took {:.4} seconds ({} blocks)".format(p[0], timing[p[1]], blocks[p[1]]), enumerate(timing))))

def main():
    create_image()

if __name__ == "__main__":
    main()
