import multiprocessing as mp
from multiprocessing import Process, Pipe
import mandelbrot
from time import time
from PIL import Image

# Get processor count
cpu_count = mp.cpu_count()

# Choose the size
width, height = (280 * 2, 300 * 2)


def compute_slice(process_index, slice_width, pipe):
    mandelbrot.set_size((width, height))

    start = time()
    result = []
    # Go through the full height of the image
    for y in range(height):
        # On the X-axis only compute my share
        for x in range(process_index * slice_width, (process_index + 1) * slice_width):
            pixel = mandelbrot.compute_pixel((x, y))
            result.append(pixel)
    end = time()

    # Send the result to the master process
    pipe.send(result)
    # Send also timing information
    pipe.send(end - start)
    # Close the communication channel
    pipe.close()


def create_image():
    output = Image.new("RGB", (width, height))

    slice_width = int(width / cpu_count)
    processes = []
    for index in range(cpu_count):
        # Create a pipe and obtain the two endpoints
        parent_pipe, child_pipe = Pipe()

        # Tell compute_slice what slice it should compute,
        # the slice size and passing it its end of the pipe
        new_process = Process(target=compute_slice, args=(index, slice_width, child_pipe))

        # Save some information for later use
        processes.append((index, new_process, parent_pipe))

        # Launch the process
        new_process.start()

    start = time()
    for process_index, process, pipe in processes:
        # Get the result from the working Process
        result = pipe.recv()

        # Also obtain the elapsed time
        time_elapsed = pipe.recv()

        # Wait for termination
        process.join()

        print("Process {} took {:.4} seconds".format(process_index, time_elapsed))

        # Write the result pixels in the appropriate locations in the image
        for pixel_index, pixel in enumerate(result):
            x = int(pixel_index % slice_width) + process_index * slice_width
            y = int(pixel_index / slice_width)
            output.putpixel((x, y), pixel)

    end = time()
    print("It took {:.4} seconds".format(end - start))

    # Save the image
    output.save("/tmp/out.png")

def main():
    create_image()

if __name__ == "__main__":
    main()
